%{

#include "bison-actions.h"

%}

// IDs de los tokens generados desde Flex:
%token ADD
%token SUB
%token MUL
%token DIV
%token POW

%token OPEN_PARENTHESIS
%token CLOSE_PARENTHESIS

%token OPEN_BRACES
%token CLOSE_BRACES
%token OPEN_BRACKETS
%token CLOSE_BRACKETS

%token FUNCTION
%token IF
%token ELSE
%token FOR
%token PRINT
%token SEMICOLON
%token COMPOSITE
%token DERIVATIVE
%token INTEGRATE
%token BETWEEN
%token AND
%token EVALUATE
%token IN

%token COLON
%token COMMA
%token GREATER
%token GREATER_EQUAL
%token LESSER
%token LESSER_EQUAL
%token ASSIGN
%token EQUAL
%token ADD_ASSIGN
%token INCREMENT
%token SUB_ASSIGN
%token DECREMENT
%token QUOTE

%token INTEGER
%token DOUBLE
%token STRING

// Reglas de asociatividad y precedencia (de menor a mayor):
%left ADD SUB
%left MUL DIV
%left POW

%%

program: expression	SEMICOLON										{ $$ = ProgramGrammarAction($1); }
	;

expression: expression ADD expression								{ $$ = AdditionExpressionGrammarAction($1, $3); }
	| expression SUB expression										{ $$ = SubtractionExpressionGrammarAction($1, $3); }
	| expression MUL expression										{ $$ = MultiplicationExpressionGrammarAction($1, $3); }
	| expression DIV expression										{ $$ = DivisionExpressionGrammarAction($1, $3); }
	| expression POW expression										{ $$ = PowerExpressionGrammarAction($1, $3); }
	| function														{ $$ = FunctionExpressionGrammarAction($1); }
	| factor														{ $$ = FactorExpressionGrammarAction($1); }
	;

boolean: string LESSER constant										{ $$ = LesserExpressionGrammarAction($1, $3); }
	| string LESSER_EQUAL constant									{ $$ = LesserEqualExpressionGrammarAction($1, $3); }
	| string GREATER constant										{ $$ = GreaterExpressionGrammarAction($1, $3); }
	| string GREATER_EQUAL constant									{ $$ = GreaterEqualExpressionGrammarAction($1, $3); }
	| string EQUAL constant											{ $$ = EqualExpressionGrammarAction($1, $3); }
	;

factor: OPEN_PARENTHESIS expression CLOSE_PARENTHESIS				{ $$ = ExpressionFactorGrammarAction($2); }
	| constant														{ $$ = ConstantFactorGrammarAction($1); }
	;
	
constant: DOUBLE													{ $$ = DoubleConstantGrammarAction($1); }
	| INTEGER														{ $$ = IntegerConstantGrammarAction($1) }
	;

function: FUNCTION string ASSIGN expression SEMICOLON				{ $$ = FunctionGrammarAction($2, $4); }
	| FUNCTION string ASSIGN expression COLON boolean SEMICOLON 	{ $$ = RestrictedFunctionGrammarAction($2, $4, $6); }
	| FUNCTION string												{ $$ = PiecewiseFunctionGrammarAction(); }
	;

string: STRING														{ $$ = StringGrammarAction($1); }
	;

piecewise: FUNCTION string ASSIGN OPEN_BRACES expression COLON boolean CLOSE_BRACES SEMICOLON

%%
